# #== src/prog/ncbi_cross_database.py ==
queries = ["BCL11A"]
in_path = "/Users/bellieny/projects/mEdit/hgvs_input.csv"
entrez_login = 'thedoudnalab@gmail.com'
db_from = 'gene'
dbto = 'clinvar'
final_df_orient = 'columns'

# #==  ==

# #==  ==
